
#include "iic.h"

/*****************************************************************************
 * @name       :void IIC_Init(void)
 * @function   :Initialize IIC
******************************************************************************/
void IIC_Init(void)
{
  //GPIO_InitTypeDef GPIO_InitStructure;

  //RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

  //GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
  //GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  //GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  //GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  //GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  //GPIO_Init(GPIOB, &GPIO_InitStructure);
  IIC_SCL_H();
  IIC_SDA_H();
}
/*****************************************************************************
 * @name       :void IIC_Start(void)
 * @function   :Generating IIC starting signal
******************************************************************************/
void IIC_Start(void)
{
  SDA_OUT();
  IIC_SDA_H();
  IIC_SCL_H();
  HAL_Delay(4);
  IIC_SDA_L();
  HAL_Delay(4);
  IIC_SCL_L();
}
/*****************************************************************************
 * @name       :void IIC_Stop(void)
 * @function   :Generating IIC stop signal
******************************************************************************/  
void IIC_Stop(void)
{
  SDA_OUT();
  IIC_SCL_L();
  IIC_SDA_L();
  HAL_Delay(4);
  IIC_SCL_H();
  IIC_SDA_H();
  HAL_Delay(4);
}
/*****************************************************************************
 * @name       :uint8_t IIC_Wait_Ack(void)
 * @function   :Wait for the response signal
 * @retvalue   :0-receive response signal successfully
				1-receive response signal unsuccessfully
******************************************************************************/ 
uint8_t IIC_Wait_Ack(void)
{
  uint8_t ucErrTime = 0;
  SDA_IN();
  IIC_SDA_H();
  HAL_Delay(1);
  IIC_SCL_H();
  HAL_Delay(1);
  while(READ_SDA)
  {
    ucErrTime ++;
    if(ucErrTime > 250)
    {
      IIC_Stop();
      return 1;
    }
  }
  IIC_SCL_L();
  return 0;
}
/*****************************************************************************
 * @name       :void IIC_Ack(void)
 * @function   :Generate ACK response signal
******************************************************************************/ 
void IIC_Ack(void)
{
  IIC_SCL_L();
  SDA_OUT();
  IIC_SDA_L();
  HAL_Delay(2);
  IIC_SCL_H();
  HAL_Delay(2);
  IIC_SCL_L();
}
/*****************************************************************************
 * @name       :void IIC_NAck(void)
 * @function   :Don't generate ACK response signal
******************************************************************************/ 	    
void IIC_NAck(void)
{
  IIC_SCL_L();
  SDA_OUT();
  IIC_SDA_H();
  HAL_Delay(2);
  IIC_SCL_H();
  HAL_Delay(2);
  IIC_SCL_L();
}	
/*****************************************************************************
 * @name       :void IIC_Send_Byte(uint8_t txd)
 * @function   :send a byte data by IIC bus
 * @parameters :txd:Data to be sent
******************************************************************************/				 				     	  
void IIC_Send_Byte(uint8_t txd)
{
  uint8_t t;
  SDA_OUT();
  IIC_SCL_L();
  for(t = 0; t < 8; t ++)
  {
    IIC_SDA =(txd & 0x80)>> 7;
    IIC_SDA_L();
    txd <<= 1;
    HAL_Delay(2);
    IIC_SCL_H();
    HAL_Delay(2);
    IIC_SCL_L();
    HAL_Delay(2);
  }
}
/*****************************************************************************
 * @name       :uint8_t IIC_Read_Byte(unsigned char ack)
 * @function   :read a byte data by IIC bus
 * @parameters :ack:0-send nACK
					1-send ACK
 * @retvalue   :Data to be read
******************************************************************************/		    
uint8_t IIC_Read_Byte(unsigned char ack)
{
  unsigned char i, receive = 0;
  SDA_IN();
  for(i = 0; i < 8; i ++ )
  {
    IIC_SCL_L();
    HAL_Delay(2);
    IIC_SCL_H();
    receive <<= 1;
    if(READ_SDA)receive ++;   
    HAL_Delay(1);
  }
  if(!ack)
  IIC_NAck();
  else
  IIC_Ack();
  return receive;
}
