
#include "lcd.h"
#include "stdlib.h"
#include <stdio.h>

_lcd_dev lcddev;

uint16_t POINT_COLOR = 0x0000, BACK_COLOR = 0xFFFF;
uint16_t devicecode;

uint16_t lcd_read(void)
{
  volatile uint16_t data;
  data = LCD -> LCD_RAM;
  return data;
}
/*****************************************************************************
 * @name       :void lcd_wr_reg(uint16_t data)
 * @function   :write an 16-bit command to the lcd screen
 * @parameters :data:command value to be written
******************************************************************************/
void lcd_wr_reg(uint16_t data)
{
  data = data;
  LCD -> LCD_REG = data;
}
/*****************************************************************************
 * @name       :void lcd_wr_data(uint16_t data)
 * @function   :write an 16-bit data to the lcd screen
 * @parameters :data:data value to be written
******************************************************************************/
void lcd_wr_data(uint16_t data)
{
  data = data;
  LCD -> LCD_RAM = data;
}
/*****************************************************************************
 * @name       :uint16_t lcd_rd_data(void)
 * @function   :read an 16-bit value from the lcd screen
 * @retvalue   :read value
******************************************************************************/
uint16_t lcd_rd_data(void)
{
  return lcd_read();
}
/*****************************************************************************
 * @name       :void lcd_writereg(uint16_t lcd_reg, uint16_t lcd_regvalue)
 * @function   :write data into registers
 * @parameters :lcd_reg:register address
                lcd_regvalue:data to be written
******************************************************************************/
void lcd_writereg(uint16_t lcd_reg, uint16_t lcd_regvalue)
{
  LCD -> LCD_REG = lcd_reg;
  LCD -> LCD_RAM = lcd_regvalue;
}
/*****************************************************************************
 * @name       :void lcd_readreg(uint16_t lcd_reg, uint16_t *rval, int n)
 * @function   :read value from specially registers
 * @parameters :lcd_reg:register address
 * @retvalue   :read value
******************************************************************************/
void lcd_readreg(uint16_t lcd_reg, uint16_t *rval, int n)
{
  lcd_wr_reg(lcd_reg);
  while(n--)
  {
    *(rval++)= lcd_rd_data();
    //HAL_Delay(100);
  }
}
/*****************************************************************************
 * @name       :void lcd_writeram_prepare(void)
 * @function   :write gram
******************************************************************************/
void lcd_writeram_prepare(void)
{
  lcd_wr_reg(lcddev.wramcmd);
}
/*****************************************************************************
 * @name       :void lcd_readram_prepare(void)
 * @function   :read gram
******************************************************************************/
void lcd_readram_prepare(void)
{
  lcd_wr_reg(lcddev.rramcmd);
}
/*****************************************************************************
 * @name       :void lcd_writedata_16bit(uint16_t data)
 * @function   :write an 16-bit command to the lcd screen
 * @parameters :data:data to be written
******************************************************************************/
void lcd_writedata_16bit(uint16_t data)
{
  LCD -> LCD_RAM = data;
}

uint16_t color_to_565(uint8_t r, uint8_t g, uint8_t b)
{
  return((r & 0xf8)<< 8)|((g & 0xfc)<< 3)|((b & 0xf8)>> 3);
}
/*****************************************************************************
 * @name       :uint16_t lcd_readdata_16bit(void)
 * @function   :read an 16-bit value from the lcd screen
 * @retvalue   :read value
******************************************************************************/
uint16_t lcd_readdata_16bit(void)
{
  uint16_t r, g, b;
  r = lcd_rd_data();
  r = lcd_rd_data();
  g = lcd_rd_data();
  b = g >> 8;
  g = r & 0xff;
  r = r >> 8;
  return color_to_565(r, g, b);
}
/*****************************************************************************
 * @name       :void lcd_drawpoint(uint16_t x, uint16_t y)
 * @function   :write a pixel data at a specified location
 * @parameters :x:the x coordinate of the pixel
                y:the y coordinate of the pixel
******************************************************************************/
void lcd_drawpoint(uint16_t x, uint16_t y)
{
  lcd_setcursor(x, y);
  lcd_writedata_16bit(POINT_COLOR);
}
/*****************************************************************************
 * @name       :uint16_t lcd_readpoint(uint16_t x, uint16_t y)
 * @function   :read a pixel color value at a specified location
 * @parameters :x:the x coordinate of the pixel
                y:the y coordinate of the pixel
 * @retvalue   :the read color value
******************************************************************************/
uint16_t lcd_readpoint(uint16_t x, uint16_t y)
{
  uint16_t color;
  if(x >= lcddev.width||y >= lcddev.height)
  {
    return 0;
  }
  lcd_setcursor(x, y);
  lcd_readram_prepare();
  color = lcd_readdata_16bit();
  return color;
}
/*****************************************************************************
 * @name       :void lcd_clear(uint16_t color)
 * @function   :full screen filled lcd screen
 * @parameters :color:filled color
******************************************************************************/
void lcd_clear(uint16_t color)
{
  unsigned int i;
  uint32_t total_point = lcddev.width *lcddev.height;
  lcd_setwindows(0, 0, lcddev.width - 1, lcddev.height - 1);
  for(i = 0; i < total_point; i++)
  {
    LCD -> LCD_RAM = color;
  }
}
/*****************************************************************************
 * @name       :void lcd_init(void)
 * @function   :initialization lcd screen
******************************************************************************/
void lcd_init(void)
{
  //************* NT35510**********//
  lcd_wr_reg(0xF000); lcd_wr_data(0x55);
  lcd_wr_reg(0xF001); lcd_wr_data(0xAA);
  lcd_wr_reg(0xF002); lcd_wr_data(0x52);
  lcd_wr_reg(0xF003); lcd_wr_data(0x08);
  lcd_wr_reg(0xF004); lcd_wr_data(0x01);
  // #AVDD
  lcd_wr_reg(0xB600); lcd_wr_data(0x34);
  lcd_wr_reg(0xB601); lcd_wr_data(0x34);
  lcd_wr_reg(0xB602); lcd_wr_data(0x34);
  lcd_wr_reg(0xB000); lcd_wr_data(0x0D);
  lcd_wr_reg(0xB001); lcd_wr_data(0x0D);
  lcd_wr_reg(0xB002); lcd_wr_data(0x0D);
  // #AVEE
  lcd_wr_reg(0xB700); lcd_wr_data(0x24);
  lcd_wr_reg(0xB701); lcd_wr_data(0x24);
  lcd_wr_reg(0xB702); lcd_wr_data(0x24);
  lcd_wr_reg(0xB100); lcd_wr_data(0x0D);
  lcd_wr_reg(0xB101); lcd_wr_data(0x0D);
  lcd_wr_reg(0xB102); lcd_wr_data(0x0D);
  // #Power Control
  // #VCL
  lcd_wr_reg(0xB800); lcd_wr_data(0x24);
  lcd_wr_reg(0xB801); lcd_wr_data(0x24);
  lcd_wr_reg(0xB802); lcd_wr_data(0x24);
  lcd_wr_reg(0xB200); lcd_wr_data(0x00);
  // #VGH
  lcd_wr_reg(0xB900); lcd_wr_data(0x24);
  lcd_wr_reg(0xB901); lcd_wr_data(0x24);
  lcd_wr_reg(0xB902); lcd_wr_data(0x24);
  lcd_wr_reg(0xB300); lcd_wr_data(0x05);
  lcd_wr_reg(0xB301); lcd_wr_data(0x05);
  lcd_wr_reg(0xB302); lcd_wr_data(0x05);
  // #VGL(LVGL)
  lcd_wr_reg(0xBA00); lcd_wr_data(0x34);
  lcd_wr_reg(0xBA01); lcd_wr_data(0x34);
  lcd_wr_reg(0xBA02); lcd_wr_data(0x34);
  // #VGL_REG(VGLO)
  lcd_wr_reg(0xB500); lcd_wr_data(0x0B);
  lcd_wr_reg(0xB501); lcd_wr_data(0x0B);
  lcd_wr_reg(0xB502); lcd_wr_data(0x0B);
  // #VGMP/VGSP:
  lcd_wr_reg(0xBC00); lcd_wr_data(0X00);
  lcd_wr_reg(0xBC01); lcd_wr_data(0xA3);
  lcd_wr_reg(0xBC02); lcd_wr_data(0X00);
  // #VGMN/VGSN
  lcd_wr_reg(0xBD00); lcd_wr_data(0x00);
  lcd_wr_reg(0xBD01); lcd_wr_data(0xA3);
  lcd_wr_reg(0xBD02); lcd_wr_data(0x00);
  // #VCOM=-0.1
  lcd_wr_reg(0xBE00); lcd_wr_data(0x00);
  lcd_wr_reg(0xBE01); lcd_wr_data(0x63);
  // #R+
  lcd_wr_reg(0xD100); lcd_wr_data(0x00);
  lcd_wr_reg(0xD101); lcd_wr_data(0x37);
  lcd_wr_reg(0xD102); lcd_wr_data(0x00);
  lcd_wr_reg(0xD103); lcd_wr_data(0x52);
  lcd_wr_reg(0xD104); lcd_wr_data(0x00);
  lcd_wr_reg(0xD105); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD106); lcd_wr_data(0x00);
  lcd_wr_reg(0xD107); lcd_wr_data(0x99);
  lcd_wr_reg(0xD108); lcd_wr_data(0x00);
  lcd_wr_reg(0xD109); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD10A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD10B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD10C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD10D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD10E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD10F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD110); lcd_wr_data(0x01);
  lcd_wr_reg(0xD111); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD112); lcd_wr_data(0x01);
  lcd_wr_reg(0xD113); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD114); lcd_wr_data(0x01);
  lcd_wr_reg(0xD115); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD116); lcd_wr_data(0x02);
  lcd_wr_reg(0xD117); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD118); lcd_wr_data(0x02);
  lcd_wr_reg(0xD119); lcd_wr_data(0x48);
  lcd_wr_reg(0xD11A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD11B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD11C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD11D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD11E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD11F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD120); lcd_wr_data(0x02);
  lcd_wr_reg(0xD121); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD122); lcd_wr_data(0x03);
  lcd_wr_reg(0xD123); lcd_wr_data(0x10);
  lcd_wr_reg(0xD124); lcd_wr_data(0x03);
  lcd_wr_reg(0xD125); lcd_wr_data(0x31);
  lcd_wr_reg(0xD126); lcd_wr_data(0x03);
  lcd_wr_reg(0xD127); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD128); lcd_wr_data(0x03);
  lcd_wr_reg(0xD129); lcd_wr_data(0x73);
  lcd_wr_reg(0xD12A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD12B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD12C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD12D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD12E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD12F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD130); lcd_wr_data(0x03);
  lcd_wr_reg(0xD131); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD132); lcd_wr_data(0x03);
  lcd_wr_reg(0xD133); lcd_wr_data(0xC1);
  // #G+
  lcd_wr_reg(0xD200); lcd_wr_data(0x00);
  lcd_wr_reg(0xD201); lcd_wr_data(0x37);
  lcd_wr_reg(0xD202); lcd_wr_data(0x00);
  lcd_wr_reg(0xD203); lcd_wr_data(0x52);
  lcd_wr_reg(0xD204); lcd_wr_data(0x00);
  lcd_wr_reg(0xD205); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD206); lcd_wr_data(0x00);
  lcd_wr_reg(0xD207); lcd_wr_data(0x99);
  lcd_wr_reg(0xD208); lcd_wr_data(0x00);
  lcd_wr_reg(0xD209); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD20A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD20B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD20C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD20D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD20E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD20F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD210); lcd_wr_data(0x01);
  lcd_wr_reg(0xD211); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD212); lcd_wr_data(0x01);
  lcd_wr_reg(0xD213); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD214); lcd_wr_data(0x01);
  lcd_wr_reg(0xD215); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD216); lcd_wr_data(0x02);
  lcd_wr_reg(0xD217); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD218); lcd_wr_data(0x02);
  lcd_wr_reg(0xD219); lcd_wr_data(0x48);
  lcd_wr_reg(0xD21A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD21B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD21C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD21D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD21E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD21F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD220); lcd_wr_data(0x02);
  lcd_wr_reg(0xD221); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD222); lcd_wr_data(0x03);
  lcd_wr_reg(0xD223); lcd_wr_data(0x10);
  lcd_wr_reg(0xD224); lcd_wr_data(0x03);
  lcd_wr_reg(0xD225); lcd_wr_data(0x31);
  lcd_wr_reg(0xD226); lcd_wr_data(0x03);
  lcd_wr_reg(0xD227); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD228); lcd_wr_data(0x03);
  lcd_wr_reg(0xD229); lcd_wr_data(0x73);
  lcd_wr_reg(0xD22A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD22B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD22C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD22D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD22E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD22F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD230); lcd_wr_data(0x03);
  lcd_wr_reg(0xD231); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD232); lcd_wr_data(0x03);
  lcd_wr_reg(0xD233); lcd_wr_data(0xC1);
  // #B+
  lcd_wr_reg(0xD300); lcd_wr_data(0x00);
  lcd_wr_reg(0xD301); lcd_wr_data(0x37);
  lcd_wr_reg(0xD302); lcd_wr_data(0x00);
  lcd_wr_reg(0xD303); lcd_wr_data(0x52);
  lcd_wr_reg(0xD304); lcd_wr_data(0x00);
  lcd_wr_reg(0xD305); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD306); lcd_wr_data(0x00);
  lcd_wr_reg(0xD307); lcd_wr_data(0x99);
  lcd_wr_reg(0xD308); lcd_wr_data(0x00);
  lcd_wr_reg(0xD309); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD30A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD30B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD30C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD30D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD30E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD30F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD310); lcd_wr_data(0x01);
  lcd_wr_reg(0xD311); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD312); lcd_wr_data(0x01);
  lcd_wr_reg(0xD313); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD314); lcd_wr_data(0x01);
  lcd_wr_reg(0xD315); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD316); lcd_wr_data(0x02);
  lcd_wr_reg(0xD317); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD318); lcd_wr_data(0x02);
  lcd_wr_reg(0xD319); lcd_wr_data(0x48);
  lcd_wr_reg(0xD31A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD31B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD31C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD31D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD31E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD31F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD320); lcd_wr_data(0x02);
  lcd_wr_reg(0xD321); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD322); lcd_wr_data(0x03);
  lcd_wr_reg(0xD323); lcd_wr_data(0x10);
  lcd_wr_reg(0xD324); lcd_wr_data(0x03);
  lcd_wr_reg(0xD325); lcd_wr_data(0x31);
  lcd_wr_reg(0xD326); lcd_wr_data(0x03);
  lcd_wr_reg(0xD327); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD328); lcd_wr_data(0x03);
  lcd_wr_reg(0xD329); lcd_wr_data(0x73);
  lcd_wr_reg(0xD32A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD32B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD32C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD32D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD32E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD32F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD330); lcd_wr_data(0x03);
  lcd_wr_reg(0xD331); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD332); lcd_wr_data(0x03);
  lcd_wr_reg(0xD333); lcd_wr_data(0xC1);
  // #R-
  lcd_wr_reg(0xD400); lcd_wr_data(0x00);
  lcd_wr_reg(0xD401); lcd_wr_data(0x37);
  lcd_wr_reg(0xD402); lcd_wr_data(0x00);
  lcd_wr_reg(0xD403); lcd_wr_data(0x52);
  lcd_wr_reg(0xD404); lcd_wr_data(0x00);
  lcd_wr_reg(0xD405); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD406); lcd_wr_data(0x00);
  lcd_wr_reg(0xD407); lcd_wr_data(0x99);
  lcd_wr_reg(0xD408); lcd_wr_data(0x00);
  lcd_wr_reg(0xD409); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD40A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD40B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD40C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD40D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD40E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD40F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD410); lcd_wr_data(0x01);
  lcd_wr_reg(0xD411); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD412); lcd_wr_data(0x01);
  lcd_wr_reg(0xD413); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD414); lcd_wr_data(0x01);
  lcd_wr_reg(0xD415); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD416); lcd_wr_data(0x02);
  lcd_wr_reg(0xD417); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD418); lcd_wr_data(0x02);
  lcd_wr_reg(0xD419); lcd_wr_data(0x48);
  lcd_wr_reg(0xD41A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD41B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD41C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD41D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD41E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD41F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD420); lcd_wr_data(0x02);
  lcd_wr_reg(0xD421); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD422); lcd_wr_data(0x03);
  lcd_wr_reg(0xD423); lcd_wr_data(0x10);
  lcd_wr_reg(0xD424); lcd_wr_data(0x03);
  lcd_wr_reg(0xD425); lcd_wr_data(0x31);
  lcd_wr_reg(0xD426); lcd_wr_data(0x03);
  lcd_wr_reg(0xD427); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD428); lcd_wr_data(0x03);
  lcd_wr_reg(0xD429); lcd_wr_data(0x73);
  lcd_wr_reg(0xD42A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD42B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD42C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD42D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD42E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD42F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD430); lcd_wr_data(0x03);
  lcd_wr_reg(0xD431); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD432); lcd_wr_data(0x03);
  lcd_wr_reg(0xD433); lcd_wr_data(0xC1);
  // #G-
  lcd_wr_reg(0xD500); lcd_wr_data(0x00);
  lcd_wr_reg(0xD501); lcd_wr_data(0x37);
  lcd_wr_reg(0xD502); lcd_wr_data(0x00);
  lcd_wr_reg(0xD503); lcd_wr_data(0x52);
  lcd_wr_reg(0xD504); lcd_wr_data(0x00);
  lcd_wr_reg(0xD505); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD506); lcd_wr_data(0x00);
  lcd_wr_reg(0xD507); lcd_wr_data(0x99);
  lcd_wr_reg(0xD508); lcd_wr_data(0x00);
  lcd_wr_reg(0xD509); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD50A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD50B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD50C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD50D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD50E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD50F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD510); lcd_wr_data(0x01);
  lcd_wr_reg(0xD511); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD512); lcd_wr_data(0x01);
  lcd_wr_reg(0xD513); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD514); lcd_wr_data(0x01);
  lcd_wr_reg(0xD515); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD516); lcd_wr_data(0x02);
  lcd_wr_reg(0xD517); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD518); lcd_wr_data(0x02);
  lcd_wr_reg(0xD519); lcd_wr_data(0x48);
  lcd_wr_reg(0xD51A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD51B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD51C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD51D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD51E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD51F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD520); lcd_wr_data(0x02);
  lcd_wr_reg(0xD521); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD522); lcd_wr_data(0x03);
  lcd_wr_reg(0xD523); lcd_wr_data(0x10);
  lcd_wr_reg(0xD524); lcd_wr_data(0x03);
  lcd_wr_reg(0xD525); lcd_wr_data(0x31);
  lcd_wr_reg(0xD526); lcd_wr_data(0x03);
  lcd_wr_reg(0xD527); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD528); lcd_wr_data(0x03);
  lcd_wr_reg(0xD529); lcd_wr_data(0x73);
  lcd_wr_reg(0xD52A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD52B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD52C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD52D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD52E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD52F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD530); lcd_wr_data(0x03);
  lcd_wr_reg(0xD531); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD532); lcd_wr_data(0x03);
  lcd_wr_reg(0xD533); lcd_wr_data(0xC1);
  // #B-
  lcd_wr_reg(0xD600); lcd_wr_data(0x00);
  lcd_wr_reg(0xD601); lcd_wr_data(0x37);
  lcd_wr_reg(0xD602); lcd_wr_data(0x00);
  lcd_wr_reg(0xD603); lcd_wr_data(0x52);
  lcd_wr_reg(0xD604); lcd_wr_data(0x00);
  lcd_wr_reg(0xD605); lcd_wr_data(0x7B);
  lcd_wr_reg(0xD606); lcd_wr_data(0x00);
  lcd_wr_reg(0xD607); lcd_wr_data(0x99);
  lcd_wr_reg(0xD608); lcd_wr_data(0x00);
  lcd_wr_reg(0xD609); lcd_wr_data(0xB1);
  lcd_wr_reg(0xD60A); lcd_wr_data(0x00);
  lcd_wr_reg(0xD60B); lcd_wr_data(0xD2);
  lcd_wr_reg(0xD60C); lcd_wr_data(0x00);
  lcd_wr_reg(0xD60D); lcd_wr_data(0xF6);
  lcd_wr_reg(0xD60E); lcd_wr_data(0x01);
  lcd_wr_reg(0xD60F); lcd_wr_data(0x27);
  lcd_wr_reg(0xD610); lcd_wr_data(0x01);
  lcd_wr_reg(0xD611); lcd_wr_data(0x4E);
  lcd_wr_reg(0xD612); lcd_wr_data(0x01);
  lcd_wr_reg(0xD613); lcd_wr_data(0x8C);
  lcd_wr_reg(0xD614); lcd_wr_data(0x01);
  lcd_wr_reg(0xD615); lcd_wr_data(0xBE);
  lcd_wr_reg(0xD616); lcd_wr_data(0x02);
  lcd_wr_reg(0xD617); lcd_wr_data(0x0B);
  lcd_wr_reg(0xD618); lcd_wr_data(0x02);
  lcd_wr_reg(0xD619); lcd_wr_data(0x48);
  lcd_wr_reg(0xD61A); lcd_wr_data(0x02);
  lcd_wr_reg(0xD61B); lcd_wr_data(0x4A);
  lcd_wr_reg(0xD61C); lcd_wr_data(0x02);
  lcd_wr_reg(0xD61D); lcd_wr_data(0x7E);
  lcd_wr_reg(0xD61E); lcd_wr_data(0x02);
  lcd_wr_reg(0xD61F); lcd_wr_data(0xBC);
  lcd_wr_reg(0xD620); lcd_wr_data(0x02);
  lcd_wr_reg(0xD621); lcd_wr_data(0xE1);
  lcd_wr_reg(0xD622); lcd_wr_data(0x03);
  lcd_wr_reg(0xD623); lcd_wr_data(0x10);
  lcd_wr_reg(0xD624); lcd_wr_data(0x03);
  lcd_wr_reg(0xD625); lcd_wr_data(0x31);
  lcd_wr_reg(0xD626); lcd_wr_data(0x03);
  lcd_wr_reg(0xD627); lcd_wr_data(0x5A);
  lcd_wr_reg(0xD628); lcd_wr_data(0x03);
  lcd_wr_reg(0xD629); lcd_wr_data(0x73);
  lcd_wr_reg(0xD62A); lcd_wr_data(0x03);
  lcd_wr_reg(0xD62B); lcd_wr_data(0x94);
  lcd_wr_reg(0xD62C); lcd_wr_data(0x03);
  lcd_wr_reg(0xD62D); lcd_wr_data(0x9F);
  lcd_wr_reg(0xD62E); lcd_wr_data(0x03);
  lcd_wr_reg(0xD62F); lcd_wr_data(0xB3);
  lcd_wr_reg(0xD630); lcd_wr_data(0x03);
  lcd_wr_reg(0xD631); lcd_wr_data(0xB9);
  lcd_wr_reg(0xD632); lcd_wr_data(0x03);
  lcd_wr_reg(0xD633); lcd_wr_data(0xC1);
  // #Enable Page0
  lcd_wr_reg(0xF000); lcd_wr_data(0x55);
  lcd_wr_reg(0xF001); lcd_wr_data(0xAA);
  lcd_wr_reg(0xF002); lcd_wr_data(0x52);
  lcd_wr_reg(0xF003); lcd_wr_data(0x08);
  lcd_wr_reg(0xF004); lcd_wr_data(0x00);
  // #RGB I/F Setting
  lcd_wr_reg(0xB000); lcd_wr_data(0x08);
  lcd_wr_reg(0xB001); lcd_wr_data(0x05);
  lcd_wr_reg(0xB002); lcd_wr_data(0x02);
  lcd_wr_reg(0xB003); lcd_wr_data(0x05);
  lcd_wr_reg(0xB004); lcd_wr_data(0x02);
  // #SDT
  lcd_wr_reg(0xB600); lcd_wr_data(0x08);
  lcd_wr_reg(0xB500); lcd_wr_data(0x50);
  // #Gate EQ
  lcd_wr_reg(0xB700); lcd_wr_data(0x00);
  lcd_wr_reg(0xB701); lcd_wr_data(0x00);
  // #Source EQ
  lcd_wr_reg(0xB800); lcd_wr_data(0x01);
  lcd_wr_reg(0xB801); lcd_wr_data(0x05);
  lcd_wr_reg(0xB802); lcd_wr_data(0x05);
  lcd_wr_reg(0xB803); lcd_wr_data(0x05);
  // #Inversion: Column inversion (NVT)
  lcd_wr_reg(0xBC00); lcd_wr_data(0x00);
  lcd_wr_reg(0xBC01); lcd_wr_data(0x00);
  lcd_wr_reg(0xBC02); lcd_wr_data(0x00);
  // #BOE's Setting(default)
  lcd_wr_reg(0xCC00); lcd_wr_data(0x03);
  lcd_wr_reg(0xCC01); lcd_wr_data(0x00);
  lcd_wr_reg(0xCC02); lcd_wr_data(0x00);
  // #Display Timing:
  lcd_wr_reg(0xBD00); lcd_wr_data(0x01);
  lcd_wr_reg(0xBD01); lcd_wr_data(0x84);
  lcd_wr_reg(0xBD02); lcd_wr_data(0x07);
  lcd_wr_reg(0xBD03); lcd_wr_data(0x31);
  lcd_wr_reg(0xBD04); lcd_wr_data(0x00);
  lcd_wr_reg(0xBA00); lcd_wr_data(0x01);
  lcd_wr_reg(0xFF00); lcd_wr_data(0xAA);
  lcd_wr_reg(0xFF01); lcd_wr_data(0x55);
  lcd_wr_reg(0xFF02); lcd_wr_data(0x25);
  lcd_wr_reg(0xFF03); lcd_wr_data(0x01);
  lcd_wr_reg(0x3500); lcd_wr_data(0x00);
  lcd_wr_reg(0x3600); lcd_wr_data(0x00);
  lcd_wr_reg(0x3a00); lcd_wr_data(0x55);
  lcd_wr_reg(0x1100);

  HAL_Delay(120);
  lcd_wr_reg(0x2900);
  lcd_wr_reg(0x2c00);

  lcd_direction(USE_HORIZONTAL);
}
/*****************************************************************************
 * @name       :void lcd_setwindows(uint16_t xstar, uint16_t ystar,uint16_t xend,uint16_t yend)
 * @function   :setting lcd display window
 * @parameters :xstar:the bebinning x coordinate of the lcd display window
				ystar:the bebinning y coordinate of the lcd display window
				xend:the endning x coordinate of the lcd display window
				yend:the endning y coordinate of the lcd display window
******************************************************************************/
void lcd_setwindows(uint16_t xstar, uint16_t ystar, uint16_t xend, uint16_t yend)
{
  lcd_wr_reg(lcddev.setxcmd);
  lcd_wr_data(xstar >> 8);
  lcd_wr_reg(lcddev.setxcmd + 1);
  lcd_wr_data(xstar & 0xff);
  lcd_wr_reg(lcddev.setxcmd + 2);
  lcd_wr_data(xend >> 8);
  lcd_wr_reg(lcddev.setxcmd + 3);
  lcd_wr_data(xend & 0xff);
  lcd_wr_reg(lcddev.setycmd);
  lcd_wr_data(ystar >> 8);
  lcd_wr_reg(lcddev.setycmd + 1);
  lcd_wr_data(ystar & 0xff);
  lcd_wr_reg(lcddev.setycmd + 2);
  lcd_wr_data(yend >> 8);
  lcd_wr_reg(lcddev.setycmd + 3);
  lcd_wr_data(yend & 0xff);
  lcd_writeram_prepare();
}
/*****************************************************************************
 * @name       :void lcd_setcursor(uint16_t xpos, uint16_t ypos)
 * @function   :set coordinate value
 * @parameters :xpos:the  x coordinate of the pixel
				ypos:the  y coordinate of the pixel
******************************************************************************/
void lcd_setcursor(uint16_t xpos, uint16_t ypos)
{
  lcd_setwindows(xpos, ypos, xpos, ypos);
}
/*****************************************************************************
 * @name       :void lcd_direction(uint8_t direction)
 * @function   :setting the display direction of lcd screen
 * @parameters :direction:0-0 degree 1-90 degree 2-180 degree 3-270 degree
******************************************************************************/
void lcd_direction(uint8_t direction)
{
  lcddev.setxcmd = 0x2a00;
  lcddev.setycmd = 0x2b00;
  lcddev.wramcmd = 0x2c00;
  lcddev.rramcmd = 0x2e00;
  switch(direction)
  {
    case 0:
    lcddev.width = LCD_W;
    lcddev.height = LCD_H;
    lcd_writereg(0x3600, 0x00);
    break;
    case 1:
    lcddev.width = LCD_H;
    lcddev.height = LCD_W;
    lcd_writereg(0x3600, (1 << 5)|(1 << 6));
    break;
    case 2:
    lcddev.width = LCD_W;
    lcddev.height = LCD_H;
    lcd_writereg(0x3600, (1 << 7)|(1 << 6));
    break;
    case 3:
    lcddev.width = LCD_H;
    lcddev.height = LCD_W;
    lcd_writereg(0x3600, (1 << 7)|(1 << 5));
    break;
    default:
    break;
  }
}
/*****************************************************************************
 * @name       :uint16_t lcd_read_id(void)
 * @function   :read id
 * @retvalue   :id value
******************************************************************************/
uint16_t lcd_read_id(void)
{
  uint16_t val;
  lcd_wr_reg(0xF000); lcd_wr_data(0x55);
  lcd_wr_reg(0xF001); lcd_wr_data(0xAA);
  lcd_wr_reg(0xF002); lcd_wr_data(0x52);
  lcd_wr_reg(0xF003); lcd_wr_data(0x08);
  lcd_wr_reg(0xF004); lcd_wr_data(0x01);
  lcd_readreg(0xC500, &lcddev.id, 1);
  lcddev.id <<= 8;
  lcd_readreg(0xC501, &val, 1);
  lcddev.id |= val;
  return lcddev.id;
}
