
#include <stdio.h>
#include "lcd.h"
#include "gui.h"
#include "test.h"
#include "touch.h"
#include "pic.h"

uint16_t ColorTab[5]={RED, GREEN, BLUE, YELLOW, BRED};
uint16_t ColornTab[8]={RED, MAGENTA, GREEN, DARKBLUE, BLUE, BLACK, LIGHTGREEN};

/*****************************************************************************
 * @name       :void drawtestpage(uint8_t *str)
 * @function   :Drawing test interface
 * @parameters :str:the start address of the Chinese and English strings
******************************************************************************/ 
void drawtestpage(uint8_t *str)
{
  lcd_clear(WHITE);
  lcd_fill(0, 0, lcddev.width - 1, 20, RED);
  lcd_fill(0, lcddev.height - 20, lcddev.width - 1, lcddev.height - 1, RED);
  POINT_COLOR = WHITE;
  gui_strcenter(0, 2, WHITE, RED, str, 16, 1);
  gui_strcenter(0, lcddev.height - 18, WHITE, RED, (uint8_t *)"http://www.free.net", 16, 1);
}
/*****************************************************************************
 * @name       :void main_test(void)
 * @function   :Drawing the main Interface of the Comprehensive Test Program
******************************************************************************/
void main_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  drawtestpage((uint8_t *)"Main_Test");
  gui_strcenter(0, 30, BLUE, WHITE, (uint8_t *)"fsmc_16bit", 16, 1);
  gui_strcenter(0, 60, BLUE, WHITE, (uint8_t *)"stm32f407zgt6", 16, 1);
  gui_strcenter(0, 90, BLUE, WHITE, (uint8_t *)"3.97\"_nt35510_480x800", 16, 1);
  gui_strcenter(0, 120, BLUE, WHITE, (uint8_t *)"27-03-2022", 16, 1);
  HAL_Delay(3000);
}
/*****************************************************************************
 * @name       :void test_color(void)
 * @function   :Color fill test(white,black,red,green,blue)
******************************************************************************/
void test_color(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, WHITE);
  show_str(20, 30, BLACK, WHITE, (uint8_t *)"BL_Test", 16, 1);
  HAL_Delay(2500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, BLACK);
  show_str(20, 30, WHITE, BLACK, (uint8_t *)"BL_Test", 16, 1);
  HAL_Delay(2500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, RED);
  show_str(20, 30, BLACK, RED, (uint8_t *)"RED", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, GREEN);
  show_str(20, 30, BLACK, GREEN, (uint8_t *)"GREEN", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, BLUE);
  show_str(20, 30, BLACK, BLUE, (uint8_t *)"BLUE", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, RED);
  show_str(20, 30, BLACK, RED, (uint8_t *)"RED", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, GREEN);
  show_str(20, 30, BLACK, GREEN, (uint8_t *)"GREEN", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, BLUE);
  show_str(20, 30, BLACK, BLUE, (uint8_t *)"BLUE", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, RED);
  show_str(20, 30, BLACK, RED, (uint8_t *)"RED", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, GREEN);
  show_str(20, 30, BLACK, GREEN, (uint8_t *)"GREEN", 16, 1);
  HAL_Delay(500);
  lcd_fill(0, 0, lcddev.width - 1, lcddev.height - 1, BLUE);
  show_str(20, 30, BLACK, BLUE, (uint8_t *)"BLUE", 16, 1);
  HAL_Delay(500);
}
/*****************************************************************************
 * @name       :void test_read(void)
 * @function   :read test
******************************************************************************/
void test_read(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  uint16_t lcd_id, color;
  uint8_t buf[10] = {0}, i;
  uint8_t cbuf[35] = {0};
  drawtestpage((uint8_t *)"Test_ID_Read");
  lcd_fill(0, 20, lcddev.width, lcddev.height - 20, WHITE);
  lcd_id = lcd_read_id();
  sprintf((char *)buf, "ID:0x%x", lcd_id);
  show_str(50, 25, BLACK, WHITE, buf, 16, 1);
  for(i = 0; i < 7; i++)
  {
    POINT_COLOR = ColornTab[i];
    lcd_drawfillrectangle(40 - 10, 55 + i * 25 - 10, 40 + 10, 55 + i * 25 + 10);
    color = lcd_readpoint(40, 55 + i * 25);
    sprintf((char *)cbuf, "Read_Color:0x%04X", color);
    show_str(40 + 20, 55 + i * 25 - 8, POINT_COLOR, WHITE, cbuf, 16, 1);
  }
  HAL_Delay(3000);
}
/*****************************************************************************
 * @name       :void display_buttonup(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
 * @function   :Drawing a 3D button
 * @parameters :x1:the bebinning x coordinate of the button
                y1:the bebinning y coordinate of the button
				x2:the ending x coordinate of the button
				y2:the ending y coordinate of the button
******************************************************************************/
void display_buttonup(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2)
{
  POINT_COLOR = GRAY;
  lcd_drawline(x1, y1, x2, y1);
  lcd_drawline(x1, y1, x1, y2);
  POINT_COLOR = RED;
  lcd_drawline(x1 + 1, y2 - 1, x2, y2 - 1);
  POINT_COLOR = GREEN;
  lcd_drawline(x1, y2, x2, y2);
  POINT_COLOR = BLUE;
  lcd_drawline(x2 - 1, y1 + 1, x2 - 1, y2);
  POINT_COLOR = BLACK;
  lcd_drawline(x2, y1, x2, y2);
}
/*****************************************************************************
 * @name       :void Menu_Test(void)
 * @function   :Drawing a 3D menu UI
******************************************************************************/
void menu_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  drawtestpage((uint8_t *)"Menu_Test");
  display_buttonup(15, 48, 305, 68);
  show_str(100, 50, BLACK, WHITE, (uint8_t *)"Picture", 16, 1);
  display_buttonup(15, 78, 305, 98);
  show_str(100, 80, GREEN, WHITE, (uint8_t *)"Audio", 16, 1);
  display_buttonup(15, 108, 305, 128);
  show_str(100, 110, BLUE, WHITE, (uint8_t *)"Video", 16, 1);
  display_buttonup(15, 138, 305, 158);
  show_str(100, 140, RED, WHITE, (uint8_t *)"Program", 16, 1);
  HAL_Delay(3000);
}
/*****************************************************************************
 * @name       :void Test_Dynamic_Num(void)
 * @function   :Dynamic number test
******************************************************************************/
void test_dynamic_num(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  uint8_t i;
  drawtestpage((uint8_t *)"Test_Dynamic_Num");
  POINT_COLOR = BLACK;
  srand(123456);
  lcd_showstring(15, 50, 16, (uint8_t *)"     HCHO:       ug/m3", 1);
  lcd_showstring(15, 70, 16, (uint8_t *)"      CO2:       ppm", 1);
  lcd_showstring(15, 90, 16, (uint8_t *)"     TVOC:       ug/m3", 1);
  lcd_showstring(15, 110, 16, (uint8_t *)"    PM2.5:       ug/m3", 1);
  lcd_showstring(15, 130, 16, (uint8_t *)"     PM10:       ug/m3", 1);
  lcd_showstring(15, 150, 16, (uint8_t *)"      TEP:       C", 1);
  lcd_showstring(15, 170, 16, (uint8_t *)"      HUM:       %", 1);
  POINT_COLOR = RED;
  for(i = 0; i < 15; i ++)
  {
    lcd_shownum(100, 50, rand()% 10000, 5, 16);
    lcd_shownum(100, 70, rand()% 10000, 5, 16);
    lcd_shownum(100, 90, rand()% 10000, 5, 16);
    lcd_shownum(100, 110, rand()% 10000, 5, 16);
    lcd_shownum(100, 130, rand()% 10000, 5, 16);
    lcd_shownum(100, 150, rand()% 10000, 5, 16);
    lcd_shownum(100, 170, rand()% 10000, 5, 16);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
    HAL_Delay(50);
  }
}
/*****************************************************************************
 * @name       :void test_fillrec(void)
 * @function   :Rectangular display and fill test
								Display red,green,blue,yellow,pink rectangular boxes in turn,
								1500 milliseconds later,
								Fill the rectangle in red,green,blue,yellow and pink in turn
******************************************************************************/
void test_fillrec(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  uint8_t i = 0;
  drawtestpage((uint8_t *)"Test_FillRec");
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  {
    POINT_COLOR = ColorTab[i];
    lcd_drawrectangle(lcddev.width / 2 - 80 +(i * 15), lcddev.height / 2 - 80 +(i * 15), lcddev.width / 2 - 80 +(i * 15)+ 60, lcddev.height / 2 - 80 +(i * 15)+ 60);
  }
  HAL_Delay(1500);
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  {
    POINT_COLOR = ColorTab[i];
    lcd_drawfillrectangle(lcddev.width / 2 - 80 +(i * 15), lcddev.height / 2 - 80 +(i * 15), lcddev.width / 2 - 80 +(i * 15)+ 60, lcddev.height / 2 - 80 +(i * 15)+ 60);
  }
  HAL_Delay(2000);
}
/*****************************************************************************
 * @name       :void test_circle(void)
 * @function   :circular display and fill test
								Display red,green,blue,yellow,pink circular boxes in turn,
								1500 milliseconds later,
								Fill the circular in red,green,blue,yellow and pink in turn
******************************************************************************/
void test_circle(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  uint8_t i = 0;
  drawtestpage((uint8_t *)"Test_Circle");
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  gui_circle(lcddev.width / 2 - 80 +(i * 25), lcddev.height / 2 - 50 +(i * 25), ColorTab[i], 30, 0);
  HAL_Delay(1500);
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  gui_circle(lcddev.width / 2 - 80 +(i * 25), lcddev.height / 2 - 50 +(i * 25), ColorTab[i], 30, 1);
  HAL_Delay(2000);
}
/*****************************************************************************
 * @name       :void test_triangle(void)
 * @function   :triangle display and fill test
								Display red,green,blue,yellow,pink triangle boxes in turn,
								1500 milliseconds later,
								Fill the triangle in red,green,blue,yellow and pink in turn
******************************************************************************/
void test_triangle(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  uint8_t i = 0;
  drawtestpage((uint8_t *)"Test_Triangle");
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  {
    POINT_COLOR = ColorTab[i];
    draw_triangel(lcddev.width / 2 - 80 +(i * 20), lcddev.height / 2 - 20 +(i * 15), lcddev.width / 2 - 50 - 1 +(i * 20), lcddev.height / 2 - 20 - 52 - 1 +(i * 15), lcddev.width / 2 - 20 - 1 +(i * 20), lcddev.height / 2 - 20 +(i * 15));
  }
  HAL_Delay(1500);
  lcd_fill(0, 20, lcddev.width - 1, lcddev.height - 20, WHITE);
  for(i = 0; i < 5; i++)
  {
    POINT_COLOR = ColorTab[i];
    fill_triangel(lcddev.width / 2 - 80 +(i * 20), lcddev.height / 2 - 20 +(i * 15), lcddev.width / 2 - 50 - 1 +(i * 20), lcddev.height / 2 - 20 - 52 - 1 +(i * 15), lcddev.width / 2 - 20 - 1 +(i * 20), lcddev.height / 2 - 20 +(i * 15));
  }
  HAL_Delay(2000);
}
/*****************************************************************************
 * @name       :void english_font_test(void)
 * @function   :English display test
******************************************************************************/
void english_font_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  drawtestpage((uint8_t *)"English_Font_Test");
  show_str(30, 30, BLACK, WHITE, (uint8_t *)"6X12:abcdefghijklmnopqrstuvwxyz0123456789", 12, 0);
  show_str(30, 45, BLACK, WHITE, (uint8_t *)"6X12:ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", 12, 1);
  show_str(30, 60, BLACK, WHITE, (uint8_t *)"6X12:~!@#$%^&*()_+{}:<>?/|-+.", 12, 0);
  show_str(30, 80, BLACK, WHITE, (uint8_t *)"8X16:abcdefghijklmnopqrstuvwxyz0123456789", 16, 0);
  show_str(30, 100, BLACK, WHITE, (uint8_t *)"8X16:ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", 16, 1);
  show_str(30, 120, BLACK, WHITE, (uint8_t *)"8X16:~!@#$%^&*()_+{}:<>?/|-+.", 16, 0);
  HAL_Delay(5000);
}
/*****************************************************************************
 * @name       :void russian_font_test(void)
 * @function   :chinese display test
******************************************************************************/
void russian_font_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  drawtestpage((uint8_t *)"Russian_Font_Test");
  show_str(30, 30, BLACK, WHITE, (uint8_t *)"16X16:abcdefghijklmnopqrstuvwxyz0123456789", 16, 0);
  show_str(30, 70, BLACK, WHITE, (uint8_t *)"16X16:ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", 16, 1);
  show_str(30, 110, BLACK, WHITE, (uint8_t *)"24X24:ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", 24, 1);
  show_str(30, 150, BLACK, WHITE, (uint8_t *)"32X32:ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", 32, 1);
  HAL_Delay(5000);
}
/*****************************************************************************
 * @name       :void pic_test(void)
 * @function   :picture display test
******************************************************************************/
void pic_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_SET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  drawtestpage((uint8_t *)"Picture_Test");
  gui_drawbmp16(50, 50, 40, 40, gImage_qq);
  gui_drawbmp16(157, 105, 40, 40, gImage_qq);
  gui_drawbmp16(264, 160, 40, 40, gImage_qq);
  gui_drawbmp16(371, 215, 40, 40, gImage_qq);
  gui_drawbmp16(478, 270, 40, 40, gImage_qq);
  gui_drawbmp16(585, 325, 40, 40, gImage_qq);
  gui_drawbmp16(692, 380, 40, 40, gImage_qq);
  show_str(50, 175, BLUE, WHITE, (uint8_t *)"p", 16, 1);
  show_str(50, 195, BLUE, WHITE, (uint8_t *)"i", 16, 1);
  show_str(50, 215, BLUE, WHITE, (uint8_t *)"n", 16, 1);
  show_str(50, 235, BLUE, WHITE, (uint8_t *)"k", 16, 1);
  show_str(50, 255, BLUE, WHITE, (uint8_t *)"O", 16, 1);
  show_str(50, 275, BLUE, WHITE, (uint8_t *)"d", 16, 1);
  HAL_Delay(3000);
}
/*****************************************************************************
 * @name       :void rotate_test(void)
 * @function   :rotate test
******************************************************************************/
 void rotate_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_SET);

  uint8_t i = 0;
  char *Direction[4]={"Rotation:0", "Rotation:90", "Rotation:180", "Rotation:270"};
  for(i = 0; i < 4; i++)
  {
    lcd_direction(i);
    drawtestpage((uint8_t *)"Rotate_Test");
    show_str(240, 30, BLACK, WHITE, (uint8_t *)Direction[i], 16, 1);
    gui_drawbmp16(100, 100, 40, 40, gImage_qq);
    HAL_Delay(1000);
    HAL_Delay(1000);
  }
  lcd_direction(USE_HORIZONTAL);
}
/*****************************************************************************
 * @name       :void touch_test(void)
 * @function   :touch test
******************************************************************************/
 void touch_test(void)
{
  HAL_GPIO_WritePin(GPIOF, LED0_Pin, GPIO_PIN_RESET);
  HAL_GPIO_WritePin(GPIOF, LED1_Pin, GPIO_PIN_RESET);

  uint16_t j = 0;
  uint16_t colorTemp = 0;
  tp_init();
  drawtestpage((uint8_t *)"Touch_Test(KEY0_Kalibrate)");
  lcd_showstring(lcddev.width - 24, 0, 16, (uint8_t *)"RST", 1);
  POINT_COLOR = BLACK;
  lcd_fill(lcddev.width - 50, 2, lcddev.width - 50 + 22, 18, BLACK);
  uint32_t i;
  while(1)
  {
    tp_dev.scan(0);
    if(tp_dev.sta & TP_PRES_DOWN)
    {
      if(tp_dev.x < lcddev.width && tp_dev.y < lcddev.height)
      {
        if(tp_dev.x >(lcddev.width - 24)&& tp_dev.y < 16)
        {
          drawtestpage((uint8_t *)"Touch_Test(KEY0_Kalibrate)");
          lcd_showstring(lcddev.width - 24, 0, 16, (uint8_t *)"RST", 1);
          POINT_COLOR = colorTemp;
          lcd_fill(lcddev.width - 50, 2, lcddev.width - 50 + 22, 18, POINT_COLOR);
        }
        else if((tp_dev.x >(lcddev.width - 50)&& tp_dev.x <(lcddev.width - 50 + 22))&& tp_dev.y < 20)
        {
          lcd_fill(lcddev.width - 50, 2, lcddev.width - 50 + 22, 18, ColorTab[j % 5]);
          POINT_COLOR = ColorTab[(j++)% 5];
          colorTemp = POINT_COLOR;
          HAL_Delay(10);
        }
        else tp_draw_big_point(tp_dev.x, tp_dev.y, POINT_COLOR);
      }
    }
    if(i++ > 0xf)
    {
      __set_FAULTMASK(1);
      NVIC_SystemReset();
    }
  }
}
